/*=============================================================================
	UnCollision.cpp: Higor's collision octree functions
=============================================================================*/


#include "XC_Engine.h"


//============================================================================//
//============================================================================//
//===                       FCollisionCacus methods                        ===//
//============================================================================//
//============================================================================//


FCheckResult* FCollisionCacus::ActorLineCheck( FMemStack& Mem, FVector End, FVector Start, FVector Extent, BYTE ExtraNodeFlags )
{
	if ( (End-Start).IsNearlyZero() )
		return ActorPointCheck( Mem, Start, Extent, ExtraNodeFlags);

	guard( FCollisionCacus::ActorLineCheck);
	FPrecomputedRay Ray( End, Start);

	//FAST QUERY! Get nodes from grid based on Start/End
	FCollisionOctree* StartNode = this;
	for ( INT j=MAX_OCTREE_DEPTH ; j>0 ; j-- )
	{
		FCollisionOctree* TestNode = DepthGrid[j].Get( Start);
		if ( !TestNode || (TestNode != DepthGrid[j].Get( End)) ) //See if both end and start are inside same node
			continue;
		if ( !Extent.IsZero() )
		{
			FBox ReducedBox = ExpandByExtent(TestNode->BoundingBox, -Extent); //Expand negatively
			if ( !FitsBox( Start, ReducedBox) || !FitsBox( End, ReducedBox) ) //Both points not inside reduced box
				continue;
		}
		StartNode = TestNode;
		break;
	}
#ifdef CACUS_DEBUG
	if ( StartNode == this )
		LineQueries++;
	else
		FastLineQueries++;
#endif
	FCheckResult* Result = StartNode->FastLineQuery( Mem, End, Start, Extent, ExtraNodeFlags, Ray, NULL);
	if ( !Result || !Result->GetNext() )
		return Result;
	INT Found = 0;
	FCheckResult* Hits[128] = {NULL};
	for ( FCheckResult* Link=Result ; Link ; Link=Link->GetNext() )
	{
		Hits[Found++] = Link;
		if ( Found >= 127 ) //Sanity break
			break;
	}
	//Sorting is done by moving elements down
	for ( INT i=1 ; i<Found ; i++ )
		if ( i>0 && (Hits[i]->Time < Hits[i-1]->Time) )
		{
			appMemswap( &Hits[i], &Hits[i-1], 4);
			if ( i > 1 )
				i -= 2; //Eventually becomes -1
		}
	for ( i=0 ; i<Found ; i++ )
		Hits[i]->Next = Hits[i+1];
	return Hits[0];
	unguard;
}

FCheckResult* FCollisionCacus::ActorPointCheck( FMemStack& Mem, FVector Location, FVector Extent, DWORD ExtraNodeFlags )
{
	guard(FCollisionCacus::ActorPointCheck);
	//FAST QUERY! Get nodes from grid based on Location
	FCollisionOctree* StartNode = this;
	for ( INT j=MAX_OCTREE_DEPTH ; j>0 ; j-- )
	{
		FCollisionOctree* TestNode = DepthGrid[j].Get( Location);
		if ( !TestNode ) //Nothing here
			continue;
		if ( !Extent.IsZero() && !FitsBox( Location, ExpandByExtent(TestNode->BoundingBox, -Extent)) ) //Point not inside reduced box
			continue;
		StartNode = TestNode;
		break;
	}
#ifdef CACUS_DEBUG
	PointQueries++;
#endif
	return StartNode->FastPointQuery( Mem, Location, Extent, ExtraNodeFlags, NULL);
	unguard;
}

FCheckResult* FCollisionCacus::ActorRadiusCheck( FMemStack& Mem, FVector Location, FLOAT Radius, DWORD ExtraNodeFlags )
{
#ifdef CACUS_DEBUG
	RadiusQueries++;
#endif
	return (FCheckResult*) RadiusQuery( Mem, Location, Radius, ExtraNodeFlags, NULL);
}

FCheckResult* FCollisionCacus::ActorEncroachmentCheck( FMemStack& Mem, AActor* Actor, FVector Location, FRotator Rotation, DWORD ExtraNodeFlags )
{
	guard( FCollisionCacus::ActorEncroachmentCheck);
	if ( BrushMover(Actor) )
	{
#ifdef CACUS_DEBUG
		EncroachmentQueries++;
#endif
		FVector OldLocation = Actor->Location;
		Actor->Location = Location;
		FBox Box = Actor->Brush->GetCollisionBoundingBox( Actor);
		FEncroachActorList* Encroaching = EncroachmentQuery( Mem, Box, NULL); //Find possible encroachers
		if ( Encroaching )
		{
			for ( FEncroachActorList* Link=Encroaching ; Link ; Link=Link->GetNext() )
				Hooked->AddActor( Link->Actor );
			FCheckResult* Result = Hooked->ActorEncroachmentCheck( Mem, Actor, Location, Rotation, ExtraNodeFlags );
			for ( Link=Encroaching ; Link ; Link=Link->GetNext() )
			{
				Hooked->RemoveActor( Link->Actor );
				Link->Actor->CollisionTag = Link->OldTag; //Restore old collision tag to prevent slow removal triggering
			}
			Actor->Location = OldLocation;
			return Result;
		}
		Actor->Location = OldLocation;
	}
	else if ( Actor->bBlockPlayers || Actor->bBlockActors )
	{
#ifdef CACUS_DEBUG
		PointQueries++;
#endif
		return PEncroachQuery( Mem, Actor, Location, NULL); //I MAY NEED TO RETRIEVE MOVERS AS WELL...
	}
	return NULL;
	unguard;
}

//Quick Add actor/mover
//Using generic primitive code results in slower addition so i had to split the
//function in two
void FCollisionCacus::AddActor( AActor *Actor )
{
	guard( FCollisionCacus::AddActor);
//	check( Actor);
	if ( !ValidCollidingActor(Actor) )
		return;
	//Use the grid no matter what
	INT j=MAX_OCTREE_DEPTH;
	if ( BrushMover( Actor) )
	{
		FBox MoverBox = Actor->Brush->GetCollisionBoundingBox( Actor);
		FVector MoverSize = MoverBox.Max - MoverBox.Min;
		while ( j>=0 )
		{
			if ( (DepthGrid[j].BoxSize.X >= MoverSize.X) && (DepthGrid[j].BoxSize.Y >= MoverSize.Y) && (DepthGrid[j].BoxSize.Z >= MoverSize.Z) ) //Fits
			{
				FCollisionOctree* TestNode = DepthGrid[j].Get( MoverBox.Max);
				if ( TestNode && (DepthGrid[j].Get( MoverBox.Min) == TestNode) )
				{
					while ( j < MAX_OCTREE_DEPTH )
					{
						if ( TestNode->HitsCenterAxes( MoverBox) )
							break; //Doesn't go in sub-box
						if ( (DepthGrid[j+1].BoxSize.X < MoverSize.X) || (DepthGrid[j+1].BoxSize.Y < MoverSize.Y) || (DepthGrid[j+1].BoxSize.Z < MoverSize.Z) )
							break; //Doesn't fit sub-box
						INT Idx = 0;
						FBox SubBox = TestNode->GetSubOctantBox( MoverBox.Max, Idx);
						if ( !TestNode->Children[Idx] )
						{
							TestNode->Children[Idx] =  new FCollisionOctree( TestNode, SubBox, TestNode->CurDepth+1, (BYTE)Idx & 0xFF);
							ChildSet( TestNode->Children[Idx] );
						}
						TestNode = TestNode->Children[Idx];
						j++;
					}
//if ( bLog )
//	debugf( NAME_Log, TEXT("ADDING TO %i WITH DEPTH %i (%i,%i,%i) to (%i,%i,%i)"), TestNode->MIndex, TestNode->CurDepth, (INT)TestNode->BoundingBox.Min.X, (INT)TestNode->BoundingBox.Min.Y, (INT)TestNode->BoundingBox.Min.Z, (INT)TestNode->BoundingBox.Max.X, (INT)TestNode->BoundingBox.Max.Y, (INT)TestNode->BoundingBox.Max.Z  );
					TestNode->BrushMovers.AddItem( Actor);
					Actor->CollisionTag = TestNode->MIndex;
					return;
				}
			}
			j--;
		}
		BrushMovers.AddItem( Actor);
		Actor->CollisionTag = 0;
		return;
	}
	else
	{
		FVector ActorSize( Actor->CollisionRadius * 2, Actor->CollisionRadius * 2, Actor->CollisionHeight * 2);
		while ( j>=0 )
		{
			if ( (DepthGrid[j].BoxSize.X >= ActorSize.X) && (DepthGrid[j].BoxSize.Y >= ActorSize.Y) && (DepthGrid[j].BoxSize.Z >= ActorSize.Z) ) //Fits
			{
				FCollisionOctree* TestNode = DepthGrid[j].Get( Actor->Location);
				if ( TestNode && FitsBox(Actor, TestNode->BoundingBox) )
				{
					while ( j < MAX_OCTREE_DEPTH )
					{
						if ( TestNode->HitsCenterAxes( Actor) || (DepthGrid[j+1].BoxSize.X < ActorSize.X) || (DepthGrid[j+1].BoxSize.Y < ActorSize.Y) || (DepthGrid[j+1].BoxSize.Z < ActorSize.Z) )
							break; //Doesn't go in sub-box, doesn't fit sub-box
						INT Idx = 0;
						FBox SubBox = TestNode->GetSubOctantBox( Actor->Location, Idx);
						if ( !TestNode->Children[Idx] )
						{
							TestNode->Children[Idx] =  new FCollisionOctree( TestNode, SubBox, TestNode->CurDepth+1, (BYTE)Idx & 0xFF);
							ChildSet( TestNode->Children[Idx] );
						}
						TestNode = TestNode->Children[Idx];
						j++;
					}
					TestNode->Actors.AddItem( Actor);
					Actor->CollisionTag = TestNode->MIndex;
//					debugf( NAME_Log, TEXT("ADDED %s TO %i WITH DEPTH %i"), Actor->GetName(), TestNode->MIndex, TestNode->CurDepth);
					return;
				}
			}
			j--;
		}
		Actors.AddItem( Actor);
		Actor->CollisionTag = 0;
//		debugf( NAME_Log, TEXT("ADDED %s TO BASE NODE"));
	}
	unguard;
}


//Quick remove actor/mover
//No sanity checks performed for speed reasons
//Does not perform memory deallocations
void FCollisionCacus::RemoveActor( AActor *Actor )
{
	guard( FCollisionCacus::RemoveActor);
	if ( ColList.IsValidIndex(Actor->CollisionTag) && ColList(Actor->CollisionTag) )
	{
		INT i=0;
		TArray<AActor*>* AArray = ((TArray<AActor*>*) &(ColList(Actor->CollisionTag)->Actors)) + BrushMover( Actor);
		INT* ArrayNum = ((INT*) AArray) + 1;
		while ( i<*ArrayNum )
		{
			if ( (*AArray)(i) == Actor )
			{
				(*AArray)(i) = AArray->Last();
#ifdef CACUS_DEBUG
				check( *ArrayNum == AArray->Num() );
#endif
				(*ArrayNum)--;
				return;
			}
			i++;
		}
	}
	if ( !Actor->bDeleteMe )
		SlowRemoveActor( Actor);
	unguard;
}

void FCollisionCacus::CheckActorNotReferenced( AActor* Actor )
{
	if ( ValidCollidingActor( Actor) )
		RemoveActor( Actor);
}

void FCollisionCacus::Tick()
{
#ifdef CACUS_DEBUG
	guard( FCollisionCacus::Tick);
#endif
	Cleanup();
#ifdef CACUS_DEBUG
	if ( DebugCycles++ > 100 )
	{
		debugf( NAME_Log, TEXT("[DEBUG] Queries-> Encroach %i, Line %i, FastLine %i, Point %i, Radius %i"), EncroachmentQueries, LineQueries, FastLineQueries, PointQueries, RadiusQueries);
		appMemzero( &EncroachmentQueries, 20);
		DebugCycles = 0;
	}
//	Hooked->Tick();
	unguard;
#endif
}

void FCollisionCacus::ChildSet( FCollisionOctree* Child)
{
	Child->Master = this;
	DepthGrid[Child->CurDepth].Set( Child);
	if ( !MinAvailable && ColList(MinAvailable) )
		Child->MIndex = ColList.AddItem( Child);
	else
	{
		Child->MIndex = MinAvailable;
		ColList(MinAvailable++) = Child;
		while ( MinAvailable < ColList.Num() )
		{
			if ( !ColList(MinAvailable) )
				return;
			MinAvailable++;
		}
		MinAvailable = 0;
	}
}

FCollisionCacus::~FCollisionCacus()
{
	CollisionCacus = NULL;
	if ( ColList.Num() > 0 )
		ColList.Empty();
	for ( INT i=0 ; i<OCTREE_GRID_ARRAY ; i++ )
		if ( DepthGrid[i].Num() > 0 )
			DepthGrid[i].Empty();
}

//============================================================================//
//============================================================================//
//===                       FCollisionOctree methods                       ===//
//============================================================================//
//============================================================================//

void FCollisionOctree::Cleanup()
{
	guard( FCollisionOctree::Cleanup);
	INT i=0;
#ifdef CACUS_DEBUG
	guard( CleanupActors);
#endif
	while ( i<Actors.Num() )
	{
		if ( !ValidCollidingActor( Actors(i)) )
			Actors.Remove( i);
		else
			i++;
	}
	i=0;
	while ( i<BrushMovers.Num() )
	{
		if ( !ValidCollidingActor( BrushMovers(i)) )
			BrushMovers.Remove( i);
		else
			i++;
	}
#ifdef CACUS_DEBUG
	unguard;
#endif
	for ( i=0 ; i<ChildCount ; i++ )
	{
		ReducedChildren[i]->Cleanup();
		if ( ReducedChildren[i]->ChildCount || ReducedChildren[i]->Actors.Num() || ReducedChildren[i]->BrushMovers.Num() )
			continue;
		if ( --(ReducedChildren[i]->CleanupTimer) == 0 )
		{
			delete ReducedChildren[i]; //Causes ChildCount--
			ReducedChildren[i] = ReducedChildren[ChildCount];
			ReducedChildren[ChildCount] = NULL; //IN UNSAFE MODE I COULD IGNORE THIS
		}
	}
	unguard;
}

//Recursive actor iteration
//Obtain actors in a specific radius
FIteratorActorList* FCollisionOctree::RadiusQuery( FMemStack& Mem, const FVector& Location, const FLOAT& Radius, const DWORD& ExtraNodeFlags, FIteratorActorList* PrevLink )
{
	guard( FCollisionOctree::RadiusQuery);
	FLOAT RSq = Radius * Radius;
	if ( (CurDepth > 0) && (!FitsBox( Location, BoundingBox.ExpandBy( Radius))
		|| (FDistSquared(CenterPoint,Location) > RSq + FDistSquared(CenterPoint,BoundingBox.Max))) )
		return PrevLink;
	for ( INT i=0 ; i<Actors.Num() ; i++ )
		if ( ValidCollidingActor(Actors(i)) && (FDistSquared( Actors(i)->Location, Location) <= RSq) )
			PrevLink = new(Mem)FIteratorActorList( PrevLink, Actors(i));
	for ( i=0 ; i<BrushMovers.Num() ; i++ )
		if ( ValidCollidingActor(BrushMovers(i)) && (FDistSquared( BrushMovers(i)->Location, Location) <= RSq) )
			PrevLink = new(Mem)FIteratorActorList( PrevLink, BrushMovers(i));
	for ( i=0 ; i<ChildCount ; i++ )
		PrevLink = ReducedChildren[i]->RadiusQuery( Mem, Location, Radius, ExtraNodeFlags, PrevLink);
	return PrevLink;
	unguard;
}

//Recursive actor iteration
//Obtain a simple actor list touching a box
//These must be cantidates for mover->actor encroachment
FEncroachActorList* FCollisionOctree::EncroachmentQuery( FMemStack& Mem, const FBox& Box, FEncroachActorList* PrevLink)
{
#ifdef CACUS_DEBUG
	guard( Deep);
#endif
	if ( CurDepth > 0 && !BoxesTouching( Box, BoundingBox) )
		return PrevLink;
	for ( INT i=0 ; i<Actors.Num() ; i++ )
		if ( ValidCollidingActor( Actors(i)) && Actors(i)->bCollideWorld )
		{
			if ( TouchesBox( Actors(i), Box) )
				PrevLink = new(Mem)FEncroachActorList( PrevLink, Actors(i));
		}
	for ( i=0 ; i<ChildCount ; i++ )
		PrevLink = ReducedChildren[i]->EncroachmentQuery( Mem, Box, PrevLink);
	return PrevLink;
#ifdef CACUS_DEBUG
	unguard;
#endif
}


//Recursive actor iteration
//Obtains actors touching a volume/point
FCheckResult* FCollisionOctree::PointQuery( FMemStack& Mem, const FVector& Location, const FVector& Extent, const DWORD& ExtraNodeFlags, FCheckResult* PrevLink)
{
	guard( Deep);
	if ( CurDepth > 0 && !FitsBox( Location, ExpandByExtent(BoundingBox, Extent)) )
		return PrevLink;
	for ( INT i=0 ; i<Actors.Num() ; i++ )
	{
		if ( !ValidCollidingActor(Actors(i)) )
			continue;
		FVector Diff = Actors(i)->Location - Location;
		if ( (Diff.Size2D() <= Actors(i)->CollisionRadius + Extent.X)
			&& (Abs(Diff.Z) <= Actors(i)->CollisionHeight + Extent.Z) )
		{
			guard( NewLink);
			PrevLink = new(Mem)FCheckResult( 1.f, PrevLink);
			PrevLink->Actor = Actors(i);
			PrevLink->Location = Actors(i)->Location;
			if ( !(PrevLink->Location - Location).IsNearlyZero() )
				PrevLink->Normal = _UnsafeNormal( Location - PrevLink->Location);
			unguard;
		}
	}
	for ( i=0 ; i<BrushMovers.Num() ; i++ )
	{
		if ( !ValidCollidingActor(BrushMovers(i)) || !BrushMovers(i)->Brush )
			continue;
		FBox MoverBox = BrushMovers(i)->Brush->GetCollisionBoundingBox(BrushMovers(i));
		if ( !Extent.IsZero() )
			MoverBox = ExpandByExtent( MoverBox, Extent);
		if ( FitsBox( Location, MoverBox ) )
		{
			FCheckResult Hit(1.f, PrevLink);
			if ( !BrushMovers(i)->Brush->PointCheck( Hit, BrushMovers(i), Location, Extent, ExtraNodeFlags) )
			{
				PrevLink = new(Mem)FCheckResult(1.f,PrevLink);
				appMemcpy( PrevLink, &Hit, sizeof( Hit) );
			}
		}
		continue;
	}
	for ( i=0 ; i<ChildCount ; i++ )
		PrevLink = ReducedChildren[i]->PointQuery( Mem, Location, Extent, ExtraNodeFlags, PrevLink);
	return PrevLink;
	unguard;
}


//Semi-recursive actor iteration
//Obtains actors touching a volume/point
FCheckResult* FCollisionOctree::FastPointQuery( FMemStack& Mem, const FVector& Location, const FVector& Extent, const DWORD& ExtraNodeFlags, FCheckResult* PrevLink)
{
	guard( FCollisionOctree::FastPointQuery);
	INT i;
	for ( FCollisionOctree* Link=this ; Link ; Link=Link->Parent )
	{	if ( Link->Actors.Num() )
		{
			for ( i=0 ; i<Link->Actors.Num() ; i++ )
			{
				if ( !ValidCollidingActor(Link->Actors(i)) )
					continue;
				FVector Diff = Link->Actors(i)->Location - Location;
				if ( (Diff.SizeSquared2D() <= Square(Link->Actors(i)->CollisionRadius + Extent.X))
					&& (Abs(Diff.Z) <= Link->Actors(i)->CollisionHeight + Extent.Z) )
				{
					PrevLink = new(Mem)FCheckResult( 1.f, PrevLink);
					PrevLink->Actor = Link->Actors(i);
					PrevLink->Location = Link->Actors(i)->Location;
					if ( !(PrevLink->Location - Location).IsNearlyZero() )
						PrevLink->Normal = _UnsafeNormal( Location - PrevLink->Location);
				}
			}
		}
		if ( Link->BrushMovers.Num() )
		{
			for ( i=0 ; i<Link->BrushMovers.Num() ; i++ )
			{
				if ( !ValidCollidingActor(Link->BrushMovers(i)) || !Link->BrushMovers(i)->Brush )
					continue;
				FBox MoverBox = Link->BrushMovers(i)->Brush->GetCollisionBoundingBox(Link->BrushMovers(i));
				if ( !Extent.IsZero() )
					MoverBox = ExpandByExtent( MoverBox, Extent);
				if ( FitsBox( Location, MoverBox ) )
				{
					FCheckResult Hit(1.f, PrevLink);
					if ( !Link->BrushMovers(i)->Brush->PointCheck( Hit, Link->BrushMovers(i), Location, Extent, ExtraNodeFlags) )
					{
						PrevLink = new(Mem)FCheckResult(1.f,PrevLink);
						appMemcpy( PrevLink, &Hit, sizeof( Hit) );
					}
				}
			}
		}
	}
	for ( i=0 ; i<ChildCount ; i++ )
		PrevLink = ReducedChildren[i]->PointQuery( Mem, Location, Extent, ExtraNodeFlags, PrevLink);
	return PrevLink;
	unguard;
}

//Recursive actor iteration
//Obtains actors touching a volume/point
FCheckResult* FCollisionOctree::PEncroachQuery( FMemStack& Mem, AActor* Actor, const FVector& Location, FCheckResult* PrevLink)
{
	guard( Deep);
	for ( INT i=0 ; i<Actors.Num() ; i++ )
	{
		if ( Actors(i)->bStatic || (Actor == Actors(i)) || !ValidCollidingActor(Actors(i)) || (!Actors(i)->bBlockActors && !Actors(i)->bBlockPlayers) )
			continue;
		if ( !Actor->bBlockActors && (!Actors(i)->bIsPawn || !Actors(i)->IsA( APlayerPawn::StaticClass())) )
			continue;
		FVector Diff = Actors(i)->Location - Location;
		if ( (Abs(Diff.Z) < Actor->CollisionHeight + Actors(i)->CollisionHeight)
			&& (Diff.SizeSquared2D() < Square( Actor->CollisionRadius + Actors(i)->CollisionRadius))  )
		{
			PrevLink = new(Mem)FCheckResult( 0.f, PrevLink);
			PrevLink->Actor = Actors(i);
			PrevLink->Location = Actors(i)->Location;
		}
	}
	for ( i=0 ; i<BrushMovers.Num() ; i++ )
	{
		if ( !ValidCollidingActor( BrushMovers(i)) || !BrushMovers(i)->Brush )
			continue;
		FBox MoverBox = BrushMovers(i)->Brush->GetCollisionBoundingBox(BrushMovers(i));
		FVector Extent = Actor->GetCylinderExtent();
		if ( !Extent.IsZero() )
			MoverBox = ExpandByExtent( MoverBox, Extent );
		if ( FitsBox( Location, MoverBox ) )
		{
			FCheckResult Hit(1.f, PrevLink);
			if ( !BrushMovers(i)->Brush->PointCheck( Hit, BrushMovers(i), Location, Extent, 0) )
			{
				PrevLink = new(Mem)FCheckResult(1.f,PrevLink);
				appMemcpy( PrevLink, &Hit, sizeof( Hit) );
			}
		}
	}
	if ( ChildCount )
	{
		FVector Extent( Actor->CollisionRadius, Actor->CollisionRadius, Actor->CollisionHeight);
		for ( i=0 ; i<ChildCount ; i++ )
			if ( FitsBox( Location, ExpandByExtent( ReducedChildren[i]->BoundingBox, Extent)) )
				PrevLink = ReducedChildren[i]->PEncroachQuery( Mem, Actor, Location, PrevLink);
	}
	return PrevLink;
	unguard;
}



//Recursive actor iteration
//Obtains actors touching a line
FCheckResult* FCollisionOctree::LineQuery
(
 FMemStack& Mem
, const FVector& End
, const FVector& Start
, const FVector& Extent
, const BYTE& ExtraNodeFlags
, FPrecomputedRay& Ray
,  FCheckResult* PrevLink
)
{
	guard( Deep);
	if ( !CurDepth || Ray.IntersectsBox(ExpandByExtent(BoundingBox, Extent)) )
	{
		INT i;
		if ( Actors.Num() )
		{
			UBOOL bVertTrace = (Ray.XAxis.X == 0.f && Ray.XAxis.Y == 0.f);
			FVector HitLocation(0,0,0);
			FVector HitNormal(0,0,0);
			for ( i=0 ; i<Actors.Num() ; i++ )
			{
				if ( !ValidCollidingActor(Actors(i)) )
					continue;
				if ( bVertTrace )
				{
					if ( !Ray.HitsVCylActor(Actors(i), Extent, &HitLocation, &HitNormal) )
						continue;
				}
				else
				{
					if ( !Ray.HitsCylActor(Actors(i), Extent, &HitLocation, &HitNormal) )
						continue;
				}
				PrevLink = new(Mem) FCheckResult( Clamp( (HitLocation-Ray.Org).Size() / Ray.Dist, 0.f, 1.f) , PrevLink);
				PrevLink->Actor = Actors(i);
				PrevLink->Location = HitLocation;
				PrevLink->Normal = HitNormal;
			}
		}
		if ( BrushMovers.Num() )
		{
			for ( i=0 ; i<BrushMovers.Num() ; i++ )
			{
				if ( !ValidCollidingActor(BrushMovers(i)) || !BrushMovers(i)->Brush )
					continue;
				FBox MoverBox = BrushMovers(i)->Brush->GetCollisionBoundingBox(BrushMovers(i));
				if ( !Extent.IsZero() )
					MoverBox = ExpandByExtent( MoverBox, Extent);
				if ( Ray.IntersectsBox( MoverBox) )
				{
					FCheckResult Hit(1.f, PrevLink);
					if ( !BrushMovers(i)->Brush->LineCheck( Hit, BrushMovers(i), End, Start, Extent, ExtraNodeFlags) )
					{
						PrevLink = new(Mem)FCheckResult(1.f,PrevLink);
						appMemcpy( PrevLink, &Hit, sizeof( Hit) );
					}
				}
			}
		}
		for ( i=0 ; i<ChildCount ; i++ )
			PrevLink = ReducedChildren[i]->LineQuery( Mem, End, Start, Extent, ExtraNodeFlags, Ray, PrevLink);
	}
	return PrevLink;
	unguard;
}

//Semi-recursive actor iteration
//Obtains actors touching a line
FCheckResult* FCollisionOctree::FastLineQuery
(
 FMemStack& Mem
, const FVector& End
, const FVector& Start
, const FVector& Extent
, const BYTE& ExtraNodeFlags
, FPrecomputedRay& Ray
,  FCheckResult* PrevLink
)
{
	guard( FCollisionOctree::FastLineQuery);
	//Count this node as well
	INT i;
	UBOOL bVertTrace = (Ray.XAxis.X == 0.f && Ray.XAxis.Y == 0.f);
	for ( FCollisionOctree* Link=this ; Link ; Link=Link->Parent )
	{
		if ( Link->Actors.Num() )
		{
			for ( i=0 ; i<Link->Actors.Num() ; i++ )
			{
				if ( !ValidCollidingActor(Link->Actors(i)) )
					continue;
				FVector HitLocation(0,0,0);
				FVector HitNormal(0,0,0);
				if ( bVertTrace )
				{
					if ( !Ray.HitsVCylActor(Link->Actors(i), Extent, &HitLocation, &HitNormal) )
						continue;
				}
				else
				{
					if ( !Ray.HitsCylActor(Link->Actors(i), Extent, &HitLocation, &HitNormal) )
						continue;
				}
				PrevLink = new(Mem) FCheckResult( Clamp( (HitLocation-Ray.Org).Size() / Ray.Dist, 0.f, 1.f) , PrevLink);
				PrevLink->Actor = Link->Actors(i);
				PrevLink->Location = HitLocation;
				PrevLink->Normal = HitNormal;
			}
		}
		if ( Link->BrushMovers.Num() )
		{
			for ( i=0 ; i<Link->BrushMovers.Num() ; i++ )
			{
				if ( !ValidCollidingActor(Link->BrushMovers(i)) || !Link->BrushMovers(i)->Brush )
					continue;
				FBox MoverBox = Link->BrushMovers(i)->Brush->GetCollisionBoundingBox(Link->BrushMovers(i));
				if ( !Extent.IsZero() )
					MoverBox = ExpandByExtent( MoverBox, Extent);
				if ( Ray.IntersectsBox( MoverBox) )
				{
					FCheckResult Hit(1.f, PrevLink);
					if ( !Link->BrushMovers(i)->Brush->LineCheck( Hit, Link->BrushMovers(i), End, Start, Extent, ExtraNodeFlags) )
					{
						PrevLink = new(Mem)FCheckResult(1.f,PrevLink);
						appMemcpy( PrevLink, &Hit, sizeof( Hit) );
					}
				}
			}
		}
	}
	for ( i=0 ; i<ChildCount ; i++ )
		PrevLink = ReducedChildren[i]->LineQuery( Mem, End, Start, Extent, ExtraNodeFlags, Ray, PrevLink);

	return PrevLink;
	unguard;
}


//Recursive actor removal
//Does not get rid of the child octree node
//Used when some dumb coders like hacking location values for no reason
//I should really add an exception handling macro here just to mess with them
FCollisionOctree* FCollisionOctree::SlowRemoveActor( AActor* Actor)
{
	if ( Actors.RemoveItem(Actor) || BrushMovers.RemoveItem(Actor) )
		return this;
	for ( INT i=0 ; i<8 ; i++ )
		if ( Children[i] )
		{
			FCollisionOctree* Result = Children[i]->SlowRemoveActor(Actor);
			if ( Result )
				return Result;
		}
	return NULL;
}

// Obtain a child box based on a position
// This doesn't do any kind of sanity checks for speed reasons
FBox FCollisionOctree::GetSubOctantBox( const FVector& Point, INT& Idx)
{
	if ( Point.X >= CenterPoint.X )
	{	if ( Point.Y >= CenterPoint.Y )
		{	if ( Point.Z >= CenterPoint.Z )
			{
				Idx = 7;
				return FBox( CenterPoint, BoundingBox.Max); //7 = (1,1,1)
			}
			Idx = 6;
			return FBox( FVector( CenterPoint.X, CenterPoint.Y, BoundingBox.Min.Z), FVector( BoundingBox.Max.X, BoundingBox.Max.Y, CenterPoint.Z) ); //6 = (1,1,0)
		}
		if ( Point.Z >= CenterPoint.Z )
		{
			Idx = 5;
			return FBox( FVector( CenterPoint.X, BoundingBox.Min.Y, CenterPoint.Z), FVector( BoundingBox.Max.X, CenterPoint.Y, BoundingBox.Max.Z) ); //5 = (1,0,1)
		}
		Idx = 4;
		return FBox( FVector( CenterPoint.X, BoundingBox.Min.Y, BoundingBox.Min.Z), FVector( BoundingBox.Max.X, CenterPoint.Y, CenterPoint.Z) );//4 = (1,0,0)
	}
	if ( Point.Y >= CenterPoint.Y )
	{	if ( Point.Z >= CenterPoint.Z )
		{
			Idx = 3;
			return FBox( FVector(BoundingBox.Min.X, CenterPoint.Y, CenterPoint.Z), FVector( CenterPoint.X, BoundingBox.Max.Y, BoundingBox.Max.Z) ); //3 = (0,1,1)
		}
		Idx = 2;
		return FBox( FVector(BoundingBox.Min.X, CenterPoint.Y, BoundingBox.Min.Z), FVector( CenterPoint.X, BoundingBox.Max.Y, CenterPoint.Z) ); //2 = (0,1,0)
	}
	if ( Point.Z >= CenterPoint.Z )
	{
		Idx = 1;
		return FBox( FVector(BoundingBox.Min.X, BoundingBox.Min.Y, CenterPoint.Z), FVector( CenterPoint.X, CenterPoint.Y, BoundingBox.Max.Z) ); //1 = (0,0,1)
	}
	Idx = 0;
	return FBox( BoundingBox.Min, CenterPoint);//0 = (0,0,0)
}

INT FCollisionOctree::GetSubOctantIdx( const FVector& Point)
{
	if ( Point.X >= CenterPoint.X )
	{	if ( Point.Y >= CenterPoint.Y )
		{	if ( Point.Z >= CenterPoint.Z )
				return 7;
			return 6;
		}
		if ( Point.Z >= CenterPoint.Z )
			return 5;
		return 4;
	}
	if ( Point.Y >= CenterPoint.Y )
	{	if ( Point.Z >= CenterPoint.Z )
			return 3;
		return 2;
	}
	if ( Point.Z >= CenterPoint.Z )
		return 1;
	return 0;
}

UBOOL FCollisionOctree::BoxesTouching( const FBox& A, const FBox& B)
{
	FVector Extent = A.Max - A.Min;
	return FitsBox( A, ExpandByExtent( B, Extent) );
}

FCollisionOctree::~FCollisionOctree()
{
	guard( Destructor);
	INT i;
	if ( Parent )
	{
#ifdef CACUS_DEBUG
		check( Parent->Children[ ChildIndex] == this);
#endif
		Parent->Children[ChildIndex] = NULL;
		Parent->ChildCount--;
	}
	if ( Master )
	{
		if ( CurDepth )
			Master->DepthGrid[CurDepth].Remove( this);
		if ( (MIndex < Master->ColList.Num()) && (Master->ColList(MIndex) == this) )
		{
			Master->ColList(MIndex) = NULL;
			if ( !Master->MinAvailable )
				Master->MinAvailable = MIndex;
			else
				Master->MinAvailable = Min( Master->MinAvailable, MIndex);
		}
	}
	if ( Actors.Num() > 0 )
		Actors.Empty();
	if ( BrushMovers.Num() > 0 )
		BrushMovers.Empty();
	for ( i=0 ; i<8 ; i++ )
		if ( Children[i] )
			delete Children[i]; //Deletion becomes recursive
	unguard;
}


//============================================================================//
//============================================================================//
//===                       FPrecomputedRay methods                        ===//
//============================================================================//
//============================================================================//

UBOOL FPrecomputedRay::HitsVCylActor( AActor* Actor, const FVector& Extent, FVector* HitLocation, FVector* HitNormal)
{
	if ( Square(Actor->Location.X - Org.X) + Square(Actor->Location.Y - Org.Y) > Square(Extent.X + Actor->CollisionRadius) )
		return 0; //Out of cylinder radius
	FLOAT AdjZ = Actor->Location.Z - Org.Z;
	FLOAT DiffZ = End.Z - Org.Z;
	if ( (AdjZ >= 0 && DiffZ <= 0) || (AdjZ <= 0 && DiffZ >= 0) )
		return 0; //Opposite directions
	if ( Abs(AdjZ)  < (Actor->CollisionHeight + Extent.Z) )
		return 0; //Start point already colliding
	if ( Abs(AdjZ) - (Actor->CollisionHeight + Extent.Z) > Abs(DiffZ) )
		return 0; //No reach
	*HitNormal = FVector(0,0,-Sign(DiffZ));
	*HitLocation = Org;
	(*HitLocation).Z = Actor->Location.Z - (Actor->CollisionHeight + Extent.Z) * Sign(AdjZ);
	return 1;
}

UBOOL FPrecomputedRay::HitsCylActor( AActor* Actor, const FVector& Extent, FVector* HitLocation, FVector* HitNormal)
{
	//Screw quick (X,Y,Z) rejects, I made the octree for this reason!!!
	//THIS HAS ASM FOR BOTH LINUX AND WINDOWS! LET'S USE IT
	FVector AdjustedActor = Actor->Location.TransformPointBy( FCoords(Org, XAxis, YAxis, FVector(0,0,1.f)) );
	FLOAT TotalSide = Extent.X + Actor->CollisionRadius;
	if ( Abs(AdjustedActor.Y) > TotalSide )
		return 0; //Out of line's X,Y trajectory
	if ( AdjustedActor.X  < -TotalSide )
		return 0; //Way behind the trace's extent, will never hit
	if ( AdjustedActor.X > (End.X-Org.X) * XAxis.X + (End.Y-Org.Y) * XAxis.Y + TotalSide ) 
		return 0; //Actor further than our trace | another DOT product optimized

	FLOAT TotalHeight = Extent.Z + Actor->CollisionHeight;
	if ( Abs(AdjustedActor.Z) > Abs(End.Z-Org.Z) + TotalHeight )
		return 0; //Actor further than Z trace

	//Trace starts inside actor
	if ( (AdjustedActor.SizeSquared2D() < Square(TotalSide)) && (Abs(AdjustedActor.Z) < TotalHeight) )
	{
		if ( AdjustedActor.X <= 0 )
			return 0; //Tracing away from actor
		*HitLocation = Org;
		if ( Abs(AdjustedActor.Z * 0.99) > TotalHeight ) //Up or down the cylinder
			*HitNormal = FVector(0,0,-Sign(AdjustedActor.Z));
		else //On side or inside
		{
			*HitNormal = _UnsafeNormal2D( Org.X - Actor->Location.X, Org.Y - Actor->Location.Y );
			*HitLocation = *HitLocation + XAxis + *HitNormal; //I have to correct this
//			debugf( NAME_Log, TEXT("Normal #1: %f %f %f, %f"), HitNormal->X, HitNormal->Y, HitNormal->Z, HitNormal->Size() );
		}
		return 1;
	}
	FLOAT XDelta = appSqrt(TotalSide*TotalSide - AdjustedActor.Y*AdjustedActor.Y);

	FVector VStart(0,0,0);
	if ( AdjustedActor.X >= XDelta ) //We can move forward and position ourselves next to the cylinder (side by side)
	{
		VStart = Dir * _appInvSqrt( Dir.X*Dir.X + Dir.Y*Dir.Y ) * (AdjustedActor.X - XDelta);
		if ( Abs(VStart.Z - AdjustedActor.Z) < TotalHeight )
		{
			*HitLocation = VStart + Org;
			*HitNormal = _UnsafeNormal2D( HitLocation->X - Actor->Location.X, HitLocation->Y - Actor->Location.Y);
			return 1;
		}
		if ( Abs(VStart.Z) > Abs(AdjustedActor.Z) ) //We passed the actor already!
			return 0;
	}
	if ( (AdjustedActor.Z) * (End.Z-Org.Z) <= 0 ) 
		return 0; //Different Z directions, never gonna hit

	FLOAT PlaneZ = AdjustedActor.Z - Sign(AdjustedActor.Z) * TotalHeight;
	VStart = Org + (End-Org) * (PlaneZ / (End.Z-Org.Z));
	if ( (VStart - Actor->Location ).SizeSquared2D() > TotalSide*TotalSide )
		return 0;
	*HitLocation = VStart;
	*HitNormal = FVector(0,0,-Sign(End.Z - Org.Z));
	return 1;
}